import java.util.Collection;

/**
 * This class represents solvers that attempt to solve problems by performing a
 * brute-force search in their state space.
 * 
 * @param <S> the type of the states of the problems that this brute-force 
 *            solver can solve.
 * 
 */
public class BruteSolver<S extends State<S>> implements Solver<S>
{

   /**
    * {@inheritDoc}
    */
   public S solve(S initial)
   {
      // If the given state is final, then just return it.
      if(initial.isFinal())
         return initial;

      // Find the states that can be reached from this one.
      for(final S s : initial.next())
      {      
         // If you locate among them, one that leads to a final state (or is
         // itself final), then just return it.
         final S n = solve(s);
         if(n != null)
            return n;
      }

      // Otherwise, if no states are reachable from the given state, or none 
      // of the reachable states lead to a final state, return null.
      return null;
   }

}
