/**
 * Solves <i>the river-crossing problem</i>.
 *
 * @see CrossTheRiverState
 *
 */
public class CrossTheRiver
{

   public static void main(String args[])
   {

      // The program expects exactly one command-line argument: the type of
      // the solver to use.
      if(args.length != 1)
      {
         System.err.println("Usage: java CrossTheRiver (random | brute)");
         System.exit(1);
      }

      // Construct the appropriate solver.
      final Solver<CrossTheRiverState> solver;
      if(args[0].equals("random"))
         solver = new RandomSolver<CrossTheRiverState>();
      else if(args[0].equals("brute"))
         solver = new BruteSolver<CrossTheRiverState>();
      else
      {
         solver = null;
         System.err.println("Unknown solver type: " + args[0]);
         System.exit(2);
      }

      // Construct the initial state of the problem.
      final CrossTheRiverState initial = new CrossTheRiverState();

      // Solve the problem.
      final CrossTheRiverState finalz = solver.solve(initial);

      // Print the solution (or a message, in case there is no solution).
      // NOTE: In this case, the states should be printed starting from the 
      // initial state and ending up to the final one.
      if(finalz == null)
         System.out.println("No solution.");
      else
         printSolution(finalz);
   }

   /**
    * Prints the solution up to the given state.
    *
    * @param s a state.
    */
   private static final void printSolution(CrossTheRiverState s)
   {
      // Print the piece of the solution that got us here.
      if(s.previous() != null)
         printSolution(s.previous());

      // And then print this state.
      System.out.println(s);
   }

}
