/**
 * Solves the <i>change-making problem</i>.
 *
 * @see MakeChangeState
 *
 */
public class MakeChange
{

   public static void main(String args[])
   {

      // The program expects exactly two command-line arguments: the type of 
      // the solver to use, and the amount to make change for.
      if(args.length != 2) 
      {
         System.err.println("Usage: java MakeChange (random | brute) <amount>");
         System.exit(1);
      }

      // Construct the appropriate solver.
      final Solver<MakeChangeState> solver;
      if(args[0].equals("random"))
         solver = new RandomSolver<MakeChangeState>();
      else if(args[0].equals("brute"))
         solver = new BruteSolver<MakeChangeState>();
      else
      {
         solver = null;
         System.err.println("Unknown solver type: " + args[0]);
         System.exit(2);
      }

      // Read the amount.
      int amount = 0;
      try 
      {
         amount = Integer.parseInt(args[1]);
         if(amount < 0)
         {
            System.err.println("Illegal amount of money: " + args[1]);
            System.exit(3);
         }
      } 
      catch(NumberFormatException nfe) 
      {
         System.err.println("Illegal number: " + args[1]);
         System.exit(4);
      }

      // Construct the initial state of the problem.
      final MakeChangeState initial = new MakeChangeState(amount);
      
      // Solve the problem.
      final MakeChangeState finalz = solver.solve(initial);

      // Print the solution (or a message, in case there is no solution).
      if(finalz == null)
         System.out.println("No solution.");
      else 
      {
         MakeChangeState s = finalz;
         while(s != null) 
         {
            System.out.println(s);
            s = s.previous();
         }
      }
   }

}
