import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * A state of <i>the change-making problem</i>.
 * <p>
 * Make change for a given amount of money using a given set of coins.
 *
 */ 
public class MakeChangeState implements State<MakeChangeState>
{
   /**
    * The coins.
    */
   private static final int[] COINS = { 1717, 103, 27, 32, 56 };

   /**
    * The state, from which this state was reached.
    */
   private MakeChangeState previous;

   /**
    * The amount left to make change for in this state.
    */
   private int amount;

   /**
    * Constructs a new state with the given amount.
    * 
    * @param amount the amount to make change for.
    */
   public MakeChangeState(int amount)
   {
      this.amount = amount;
   }

   /**
    * Constructs a new state with the given amount and the given previous state.
    *
    * @param amount the amount to make change for.
    * @param previous the previous state of the new state.
    */
   private MakeChangeState(int amount, MakeChangeState previous)
   {
      this.amount = amount;
      this.previous = previous;
   }

   /**
    * {@inheritDoc}
    */
   public Collection<MakeChangeState> next()
   {
      final List<MakeChangeState> n = new ArrayList<MakeChangeState>();
      for(final int coin : COINS)
         if(coin <= amount)
            n.add(new MakeChangeState(amount - coin, this));
      return n;
   }

   /**
    * {@inheritDoc}
    */
   public MakeChangeState previous()
   {
      return previous;
   }

   /**
    * Determines whether this state is final, which in this case implies that 
    * the amount to make change for is {@code 0}.
    */
   public boolean isFinal()
   {
      return amount == 0;
   }
   
   /**
    * Returns a string representation of this state, which consists only of the 
    * amount to make change for.
    */ 
   public String toString()
   {
      return String.valueOf(amount);
   }

}
