/**
 * A solver of problems that can be modelled as state spaces.
 * 
 * @param <S> the type of the states of the problems that this solver can solve.
 *
 * @see State
 *
 */
public interface Solver<S extends State>
{

   /**
    * Solves a problem by searching its state space, starting from the given 
    * initial state. The search stops, as soon as a final state was reached.
    * 
    * @param initial the state to start the search.
    * @return the final state that was reached. 
    */ 
   public S solve(S initial);

}
